/***************************************************************************
 *   Copyright (C) 2005 by Joel R. Voss                                    *
 *   jvoss@myuw.net                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ROBOTCAT_H_
#define ROBOTCAT_H_

#include "neural.h"

#ifndef TYPES_H_
#define TYPES_H_
typedef unsigned char  BYTE;	// Hoping in all cases that byte  is  8-bits.
typedef unsigned short WORD;	// Hoping in all cases that short is 16-bits.
typedef   signed short SHORT;	// Hoping in all cases that short is 16-bits.

#endif // TYPES_H_

// 4 bytes
struct Vec2 {
	WORD x, y;	// cm, max 65536 values
};
typedef struct Vec2 Vec2;

// 8 bytes
struct Path {
	SHORT theta;	// degrees
	WORD  distance;	// cm
	Vec2  position;	// struct
};
typedef struct Path Path;

// 1 byte
enum MemType {
	error,
	start,
	temperature,
	motion,
	audio,
	good,
	bad,
	food,
	sleep,
	enemy,
	friendly,
	human
	// etc
};
typedef enum MemType MemType;

struct Memory;
// 8 bytes
// Actual: 16 bytes
struct Memory {
	Vec2 position;	// struct
	MemType type;	// enum
	BYTE value;	// magnitude of type.
	struct Memory *next_memory; // Single linked list.
};
typedef struct Memory Memory;

void Memory_init(Memory *memory, WORD x, WORD y, MemType type, BYTE value);
void Memory_add(Memory *memory, Memory *child);
void Memory_destroy(Memory *memory);

struct RobotCat {
	WORD  id;
	char *name;			// Name of the 
	Memory knowledge;		// Single Linked List of memories.
	NeuralNetwork recognition;	// A Neural Network.
	WORD current_x, current_y;	// Current x y coordinates.
};
typedef struct RobotCat RobotCat;

void RobotCat_init(RobotCat *cat, int inputs, int layers);
void RobotCat_destroy(RobotCat *cat);

// Create a memory.
void RobotCat_input(RobotCat *cat, MemType type, BYTE value);
// Use the current state of the world to continue.
void RobotCat_update(RobotCat *cat);

// Built-in Action Algorithms?
void RobotCat_explore(RobotCat *cat);
void RobotCat_seek(RobotCat *cat);
void RobotCat_avoid(RobotCat *cat);
void RobotCat_communicate(RobotCat *cat);


#endif // ROBOTCAT_H_

